// 
// Copyright (C) 2008, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
// Updated by ThatManFromTexas to change colors above and below zero Jan 2011
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Linear Regression Slope
	/// </summary>
	[Description("Linear Regression Slope")]
	public class LinRegSlopeTMFT : Indicator
	{
		#region Variables
		private int					period	= 14;
		private Color		upColor    = Color.Lime; 
		private Color		dnColor    = Color.Red;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Orange, "LinRegSlopeTMFT"));
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			double	sumX	= (double) Period * (Period - 1) * 0.5;
			double	divisor = sumX * sumX - (double) Period * Period * (Period - 1) * (2 * Period - 1) / 6;
			double	sumXY	= 0;

			for (int count = 0; count < Period && CurrentBar - count >= 0; count++)
				sumXY += count * Input[count];
				
			Value.Set(((double) Period * sumXY - sumX * SUM(Inputs[0], Period)[0]) / divisor);
			
			if(Value[0]>0)
			PlotColors[0][0] = UpColor;
			
			else 
				
				PlotColors[0][0] = DnColor;
		}

		#region Properties
		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculations")]
		[GridCategory("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(2, value); }
		}
		
		[XmlIgnore()]
        [Description(" Up Color")]
        [Category("Color")]
		[Gui.Design.DisplayNameAttribute("01. Up Color")]
        public Color UpColor
        {
            get { return upColor; }
            set { upColor = value; }
        }
		[Browsable(false)]
		public string UpColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(UpColor); }
			set { UpColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("CCI Down Color")]
        [Category("Color")]
		[Gui.Design.DisplayNameAttribute("02. CCI Down Color")]
        public Color DnColor
        {
            get { return dnColor; }
            set { dnColor = value; }
        }
		[Browsable(false)]
		public string DnColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(DnColor); }
			set { DnColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private LinRegSlopeTMFT[] cacheLinRegSlopeTMFT = null;

        private static LinRegSlopeTMFT checkLinRegSlopeTMFT = new LinRegSlopeTMFT();

        /// <summary>
        /// Linear Regression Slope
        /// </summary>
        /// <returns></returns>
        public LinRegSlopeTMFT LinRegSlopeTMFT(int period)
        {
            return LinRegSlopeTMFT(Input, period);
        }

        /// <summary>
        /// Linear Regression Slope
        /// </summary>
        /// <returns></returns>
        public LinRegSlopeTMFT LinRegSlopeTMFT(Data.IDataSeries input, int period)
        {
            if (cacheLinRegSlopeTMFT != null)
                for (int idx = 0; idx < cacheLinRegSlopeTMFT.Length; idx++)
                    if (cacheLinRegSlopeTMFT[idx].Period == period && cacheLinRegSlopeTMFT[idx].EqualsInput(input))
                        return cacheLinRegSlopeTMFT[idx];

            lock (checkLinRegSlopeTMFT)
            {
                checkLinRegSlopeTMFT.Period = period;
                period = checkLinRegSlopeTMFT.Period;

                if (cacheLinRegSlopeTMFT != null)
                    for (int idx = 0; idx < cacheLinRegSlopeTMFT.Length; idx++)
                        if (cacheLinRegSlopeTMFT[idx].Period == period && cacheLinRegSlopeTMFT[idx].EqualsInput(input))
                            return cacheLinRegSlopeTMFT[idx];

                LinRegSlopeTMFT indicator = new LinRegSlopeTMFT();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Period = period;
                Indicators.Add(indicator);
                indicator.SetUp();

                LinRegSlopeTMFT[] tmp = new LinRegSlopeTMFT[cacheLinRegSlopeTMFT == null ? 1 : cacheLinRegSlopeTMFT.Length + 1];
                if (cacheLinRegSlopeTMFT != null)
                    cacheLinRegSlopeTMFT.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheLinRegSlopeTMFT = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Linear Regression Slope
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.LinRegSlopeTMFT LinRegSlopeTMFT(int period)
        {
            return _indicator.LinRegSlopeTMFT(Input, period);
        }

        /// <summary>
        /// Linear Regression Slope
        /// </summary>
        /// <returns></returns>
        public Indicator.LinRegSlopeTMFT LinRegSlopeTMFT(Data.IDataSeries input, int period)
        {
            return _indicator.LinRegSlopeTMFT(input, period);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Linear Regression Slope
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.LinRegSlopeTMFT LinRegSlopeTMFT(int period)
        {
            return _indicator.LinRegSlopeTMFT(Input, period);
        }

        /// <summary>
        /// Linear Regression Slope
        /// </summary>
        /// <returns></returns>
        public Indicator.LinRegSlopeTMFT LinRegSlopeTMFT(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.LinRegSlopeTMFT(input, period);
        }
    }
}
#endregion
